@@ -2,12 +2,12 @@
 shapefile.py
 Provides read and write support for ESRI Shapefiles.
 author: jlawhead<at>geospatialpython.com
-date: 2017/04/29
-version: 1.2.11
+date: 2017/08/24
+version: 1.2.12
 Compatible with Python versions 2.7-3.x
 """
 
-__version__ = "1.2.11"
+__version__ = "1.2.12"
 
 from struct import pack, unpack, calcsize, error, Struct
 import os
@@ -16,6 +16,7 @@ import time
 import array
 import tempfile
 import itertools
+import io
 from datetime import date
 
 #
@@ -238,23 +239,35 @@ class Reader:
         if "shp" in kwargs.keys():
             if hasattr(kwargs["shp"], "read"):
                 self.shp = kwargs["shp"]
-                if hasattr(self.shp, "seek"):
+                # Copy if required
+                try:
                     self.shp.seek(0)
+                except (NameError, io.UnsupportedOperation):
+                    self.shp = io.BytesIO(self.shp.read())
             if "shx" in kwargs.keys():
                 if hasattr(kwargs["shx"], "read"):
                     self.shx = kwargs["shx"]
-                    if hasattr(self.shx, "seek"):
+                    # Copy if required
+                    try:
                         self.shx.seek(0)
+                    except (NameError, io.UnsupportedOperation):
+                        self.shx = io.BytesIO(self.shx.read())
         if "dbf" in kwargs.keys():
             if hasattr(kwargs["dbf"], "read"):
                 self.dbf = kwargs["dbf"]
-                if hasattr(self.dbf, "seek"):
+                # Copy if required
+                try:
                     self.dbf.seek(0)
+                except (NameError, io.UnsupportedOperation):
+                    self.dbf = io.BytesIO(self.dbf.read())
         if self.shp or self.dbf:        
             self.load()
         else:
             raise ShapefileException("Shapefile Reader requires a shapefile or file-like object.")
 
+
+
+
     def load(self, shapefile=None):
         """Opens a shapefile from a filename or file-like
         object. Normally this method would be called by the
@@ -509,11 +522,19 @@ class Reader:
                         #not parseable as float, set to None
                         value = None
                 else:
+                    # force to int
                     try:
-                        value = int(value)
+                        # first try to force directly to int.
+                        # forcing a large int to float and back to int
+                        # will lose information and result in wrong nr.
+                        value = int(value) 
                     except ValueError:
-                        #not parseable as int, set to None
-                        value = None
+                        # forcing directly to int failed, so was probably a float.
+                        try:
+                            value = int(float(value))
+                        except ValueError:
+                            #not parseable as int, set to None
+                            value = None
             elif typ == 'D':
                 # date: 8 bytes - date stored as a string in the format YYYYMMDD.
                 if value.count(b('0')) == len(value):  # QGIS NULL is all '0' chars
@@ -932,8 +953,18 @@ class Writer:
                     if value in MISSING:
                         value = str("*"*size) # QGIS NULL
                     elif not deci:
+                        # force to int
+                        try:
+                            # first try to force directly to int.
+                            # forcing a large int to float and back to int
+                            # will lose information and result in wrong nr.
+                            value = int(value) 
+                        except ValueError:
+                            # forcing directly to int failed, so was probably a float.
+                            value = int(float(value))
                         value = format(value, "d")[:size].rjust(size) # caps the size if exceeds the field size
                     else:
+                        value = float(value)
                         value = format(value, ".%sf"%deci)[:size].rjust(size) # caps the size if exceeds the field size
                 elif fieldType == "D":
                     # date: 8 bytes - date stored as a string in the format YYYYMMDD.
